# coding=UTF-8
import json

class ESRData:

	def __init__(self):
		self.tick_per_second = 6000000.0 #System timer 6 MHz
		self.interval_ticks = None #int, full time in tick
		self.values = [] #int array, value range [0..4096)
		self.resistor_index = None #int    VoltageUp1, VoltageUp2 or VoltageUp3
		self.resistor_value = None #float, Om
		self.zero_value = 0

	def makeFakeData(self):
		self.interval_ticks = 1234
		self.resistor_index = 2
		self.resistor_value = 220

		self.values = []
		for i in xrange(300):
			self.values.append(i)
		pass

	def save(self, filename):
		data = {
				'tick_per_second':self.tick_per_second, 
				'interval_ticks':self.interval_ticks,
				'values':self.values,
				'resistor_index':self.resistor_index,
				'resistor_value':self.resistor_value,
				'zero_value':self.zero_value
			}

		with open(filename, "w") as f:
			json.dump(data, f,  indent=1, separators=(',', ': '))
		pass

	def load(self, filename):
		with open(filename, "r") as f:
			data = json.load(f)

		self.tick_per_second = data['tick_per_second']
		self.interval_ticks = data['interval_ticks']
		self.values = data['values']
		self.resistor_index = data['resistor_index']
		self.resistor_value = data['resistor_value']
		self.zero_value = data['zero_value']
		pass

	def getVoltagesMillivolts(self):
		mul = 15.0/208.0
		return [ (x-self.zero_value)*mul for x in self.values]

	def getTimePerSampleMicroseconds(self):
		return self.interval_ticks*1e6/self.tick_per_second/len(self.values)

	def getTimesMicroseconds(self):
		mul = self.getTimePerSampleMicroseconds()
		return [x*mul for x in range(len(self.values))]
