# coding=UTF-8
import usb
import time
import datetime
import os
import esr_data
import struct

class UsbDriver:
    dev = None
    Vcc = 3.3 #Volts

    VoltageOff = 0
    VoltageZero = 1
    VoltageUp1 = 2 #220 om 14 mA
    VoltageUp2 = 3 #10 kom 0.33 mA
    VoltageUp3 = 4 #470 ком 7 mkA
    ResistorValues = {VoltageOff:1e6, VoltageZero:1e6, VoltageUp1:220.0, VoltageUp2:10e3, VoltageUp3:470e3}

    COMMAND_SET_VOLTAGE = 1
    COMMAND_GET_ADC = 2
    COMMAND_CONVERSION = 3
    COMMAND_GET_RESULT = 4
    COMMAND_SOUND = 5

    data = esr_data.ESRData()

    def findDevice(self):
        self.dev = usb.core.find(idVendor=0x16C0, idProduct=0x05DC)

        if self.dev is None:
        	raise ValueError('Device not found')
        else:
            print "Device found"
        self.dev.set_configuration()
        pass

    def printEndpoints(self):
        for cfg in self.dev:
            for i in cfg:
                print "interface=", i.bInterfaceNumber
                for e in i:
                    print e.bEndpointAddress
                    #printEndpoint(e)
        pass

    def readAll(self):
        time.sleep(0.1)
        while True:
            try:
                data = self.dev.read(129, 128, interface=1, timeout=50)
                if len(data)==0:
                    break
                print "readb=", data
                print "read=", data.tostring()
            except usb.core.USBError:
                break
        pass

    def readOne(self):
        time.sleep(0.1)
        try:
            data = self.dev.read(129, 128, interface=1, timeout=50)
            if len(data)==0:
                print "Read empty"
            else:
                print "readb=", data
        except usb.core.USBError:
            print "Read USB error"
        pass

    def readInt(self):
        data = self.dev.read(129, 128, interface=1, timeout=50)
        if len(data)!=4:
            print "Fail read int len="+str(len(data))
        else:
            print data[0]+data[1]*0x100+data[2]*0x10000+data[3]*0x1000000

    def readCommand(self):
        time.sleep(0.05)
        try:
            data = self.dev.read(129, 128, interface=1, timeout=50)
            if len(data)==0:
                print "Read empty"
                return
            else:
                print "readb=", data
        except usb.core.USBError:
            print "Read USB error"
            return

        if data[0]==self.COMMAND_SET_VOLTAGE:
            print "Voltage channel"+str(data[1]) 
        elif data[0]==self.COMMAND_GET_ADC:
            v = data[1]+256*data[2]
            print "voltage="+str(v*3.3/4096.0)
        elif data[0]==self.COMMAND_CONVERSION:
            print "Start conversion"
            self.data.interval_ticks = struct.unpack_from('i',data,1)[0]
            self.samples_count = struct.unpack_from('i',data,5)[0]
            print "dT=", self.data.interval_ticks
            print "samples=", self.samples_count
        else:
            print "Unknown command="+str(data[0])
        pass

    def readConversionData(self):
        self.dev.write(3, [self.COMMAND_GET_RESULT], interface=1)
        samples_readed = 0
        #print "readConversionData"
        while samples_readed<self.samples_count:
            data = self.dev.read(129, 128, interface=1, timeout=50)
            #print data
            if len(data)==0:
                print "readConversionData fail samples_readed="+str(samples_readed)+" samples_count"+str(self.samples_count)
                break

            for ix in xrange(0, len(data), 2):
                self.data.values.append(struct.unpack_from('h', data, ix)[0])
                samples_readed+=1

        pass

    def conversion(self, voltage = VoltageUp3, sleep_time=0.01):
        print "Conversion started."
        self.data = esr_data.ESRData()
        self.dev.write(3, [self.COMMAND_SET_VOLTAGE, self.VoltageZero], interface=1)        
        self.readCommand()
        time.sleep(0.1)
        self.dev.write(3, [self.COMMAND_CONVERSION, voltage], interface=1)
        self.readCommand()
        time.sleep(sleep_time)
        self.readConversionData()
        #print self.data.values
        self.data.resistor_index = voltage
        self.data.resistor_value = self.ResistorValues[voltage]

        pass

    def printEndpoint(self, e):
        print "Endpoint:"
        print "bLength=", e.bLength
        print "bDescriptorType=", e.bDescriptorType
        print "bEndpointAddress=", e.bEndpointAddress
        print "bmAttributes=", e.bmAttributes
        print "wMaxPacketSize=", e.wMaxPacketSize
        print "bInterval=", e.bInterval
        print "bRefresh=", e.bRefresh
        print "bSynchAddress=", e.bSynchAddress
        pass

    def sound(self, filename="out.uint16", max_len=None, sum_samples=4):
        """
        sum_samples=4 22050 samples per second
        """
        print "write=",self.dev.write(3, [self.COMMAND_SET_VOLTAGE, self.VoltageUp3], interface=1)
        time.sleep(0.1)
        with open(filename, "wb") as file:
            print "write=", self.dev.write(3, [self.COMMAND_SOUND, sum_samples], interface=1)
            summary_len = 0
            prev_summary_len = 0
            start_time=time.time()
            while True:
                data = self.dev.read(129, 128, interface=1, timeout=50)
                if len(data)>0:
                    file.write(data)
                    summary_len += len(data)
                    if max_len and summary_len>=max_len:
                        break
                    if summary_len>= prev_summary_len+10000:
                        prev_summary_len = summary_len
                        print summary_len
                else:
                    time.sleep(0.1)
                    print '.'
            sum_time = time.time() - start_time
            print "time=", sum_time, " samples_per_second=", summary_len/2/sum_time
        pass        

def main():
    findDevice()

    if False:
        #readAll()
        print "write=",dev.write(3, "Hello, World!1KK", interface=1)
        readAll()
        #readAll()
        print "write=",dev.write(3, "0AXB", interface=1)
        readAll()
        #time.sleep(1)
        print "write=",dev.write(3, "B0RT", interface=1)
        readAll()
        print "write=",dev.write(3, "CMNK", interface=1)
        readAll()
        print "write=",dev.write(3, "DMNK", interface=1)
        #time.sleep(1)
        readAll()

    if False:
        readAll()
        print "write=",dev.write(3, "0", interface=1)
        readAll()
        print "write=",dev.write(3, "1", interface=1)
        readAll()

    if False:
        #print "write=",dev.write(3, "Hello, World!1KK", interface=1)
        #print "write=",dev.write(3, "Hello, World!2KK", interface=1)
        for i in xrange(1,10):
            #print i
            readInt()
    if False:
        readOne()
        i = VoltageUp2
        print "write=",dev.write(3, [COMMAND_SET_VOLTAGE, VoltageUp1], interface=1)
        readCommand()
        print "write=",dev.write(3, [COMMAND_GET_ADC], interface=1)
        readCommand()
        time.sleep(1)
        readOne()

    if False:
        readOne()
        print "write=",dev.write(3, [COMMAND_SET_VOLTAGE, VoltageZero], interface=1)
        readCommand()
        print "write=",dev.write(3, [COMMAND_CONVERSION, VoltageUp3], interface=1)
        readCommand()
        time.sleep(1)
        readConversionData()

    pass

if __name__ == "__main__":
    main()

