# -*- coding: UTF-8 -*-
# balmer@inbox.ru 2013 ESR Demo
import sys, os, csv
from PyQt4 import QtCore, QtGui

import matplotlib
import time
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.figure import Figure
from esr import usb_driver, esr_data
from esr.esr_math import mmk, fineTuning, fexp


class Form(QtGui.QMainWindow):
    def __init__(self, parent=None):
        super(Form, self).__init__(parent)
        self.setWindowTitle('Balmer ESR Meter')

        self.driver = usb_driver.UsbDriver()
        self.zero_value = None
        self.Czero = 739e-6 #uf
        #self.Czero = 0 #uf

        self.create_menu()
        self.create_main_frame()
        pass        

    def load_file(self, filename=None):
        filename = QtGui.QFileDialog.getOpenFileName(self,
            'Open a data file', '.', 'JSON files (*.json);;All Files (*.*)')
        
        if filename:
            self.driver.data.load(filename)
            self.on_show()

    def save_file(self, filename=None):
        filename = QtGui.QFileDialog.getSaveFileName(self,
            'Save a data file', '.', 'JSON files (*.json);;All Files (*.*)')

        if filename:
            self.driver.data.save(filename)
        pass
    
    def plot_data(self):
        voltage = self.driver.data.getVoltagesMillivolts()
        time = self.driver.data.getTimesMicroseconds()
        self.axes.plot(time, voltage)

        self.plot_linear(time, voltage)

        pass

    def calculate_linear(self, xlist, ylist):
        R = self.driver.data.resistor_value
        Vcc_mV = self.driver.Vcc*1000.0

        if len(ylist)>200:
            ymax=50
        else:
            ymax=290

        if R>20e3:
            ymin = 5
        else:
            ymin = 0.5
            
        nend = len(ylist)
        nstart = 1
        #ищем максимальное значение
        for i in xrange(0, len(ylist)):
            if ylist[i]<ymin:
                nstart = i
            if ylist[i]>ymax:
                nend = i
                break

        if nstart+1<nend:
            xlist = xlist[nstart:nend]
            ylist = ylist[nstart:nend]

        (a,b) = mmk(xlist, ylist)
        I = Vcc_mV/R #mA
        if b>1e-4:
            C = I/b
        else:
            C = 0
        return C

    def plot_linear(self, xlist, ylist):
        R = self.driver.data.resistor_value
        Vcc_mV = self.driver.Vcc*1000.0

        if len(ylist)>200:
            ymax=50
        else:
            ymax=290

        if R>20e3:
            ymin = 5
        else:
            ymin = 0.5
            
        nend = len(ylist)
        nstart = 1
        #ищем максимальное значение
        for i in xrange(0, len(ylist)):
            if ylist[i]<ymin:
                nstart = i
            if ylist[i]>ymax:
                nend = i
                break
        print "nstart=", nstart, "  R=", R

        if nstart+1<nend:
            xlist = xlist[nstart:nend]
            ylist = ylist[nstart:nend]

        (a,b) = mmk(xlist, ylist)

        x0 = xlist[0]
        x1 = xlist[-1]
        y0 = a+b*x0
        y1 = a+b*x1
        print "a="+str(a), "b="+str(b)
        I = Vcc_mV/R #mA
        is_resistor = (b<1e-4)
        if is_resistor:
            self.axes.plot([x0, x1], [y0, y1], 'r')
            ESR = a/I
            ESRstr = "{:3.2f}".format(ESR)
            title = "R="+ESRstr+" Om"
            self.axes.set_title(title)
            self.C = 0
        else:
            C = I/b
            ESR = a/I
            print "ESR="+str(ESR), "C="+str(C)

            if R<20e3:
                (a1,b1)=fineTuning(xlist, ylist, a, C*R, Vcc_mV)
                self.axes.plot(xlist, [fexp(x, a1, b1, Vcc_mV) for x in xlist], 'r')
                C = b1/R
                ESR = a1/I
            else:
                x0 = xlist[0]
                x1 = xlist[-1]
                y0 = a+b*x0
                y1 = a+b*x1
                self.axes.plot([x0, x1], [y0, y1], 'r')
            #self.axes.plot(xlist, [fexp(x, a, C*R, Vcc_mV) for x in xlist], 'g')
            #mul = self.driver.data.getTimePerSampleMicroseconds()
            #xlist = [x*mul*20 for x in range(1000)]
            self.C = C


            if C>=1:
                Cstr = "{:3.2f} uF".format(C)
            elif C>=1e-3:
                Cstr = "{:3.2f} nF".format((C-self.Czero)*1e3)
            else:
                print "C={:3.1f} pF".format((C)*1e6), "Czero={:3.1f} pF".format((self.Czero)*1e6)
                Cstr = "{:3.1f} pF".format((C-self.Czero)*1e6)

            ESRstr = "{:3.2f}".format(ESR)

            if ESR>0:
                title = "C="+Cstr+ " ESR="+ESRstr+" Om"
            else:
                title = "C="+Cstr
            self.axes.set_title(title)

        pass


    def on_show(self):
        print "Show"
        self.axes.clear()        
        self.axes.grid(True)
        has_series = False
        
        self.axes.set_ylabel('mV')
        self.axes.set_xlabel('uS')
        self.plot_data()
        #self.axes.plot(range(len(series)), series, 'o-', label=name)
        
        if has_series and self.legend_cb.isChecked():
            self.axes.legend()
        self.canvas.draw()

    def on_about(self):
        msg = __doc__
        QMessageBox.about(self, "About the demo", msg.strip())
    
    def create_main_frame(self):
        self.main_frame = QtGui.QWidget()
        
        plot_frame = QtGui.QWidget()
        
        self.dpi = 100
        self.fig = Figure((6.0, 4.0), dpi=self.dpi)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        
        self.axes = self.fig.add_subplot(111)
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)
        
        self.connect_button = QtGui.QPushButton("Connect")
        self.connect_button.clicked.connect(self.OnConnectButton)
        self.connect_state = QtGui.QLabel("Disconnected")
        self.connect_state.setStyleSheet("QLabel { color : gray; }");

        connect_hbox = QtGui.QHBoxLayout()
        connect_hbox.addWidget(self.connect_button)
        connect_hbox.addWidget(self.connect_state)
        connect_hbox.addStretch(1)

        spin_label_resistor = QtGui.QLabel('Pullup')

        self.resistor_combo_box = QtGui.QComboBox()
        self.resistor_combo_box.addItem("220 om", QtCore.QVariant(self.driver.VoltageUp1))
        self.resistor_combo_box.addItem("10 kom", QtCore.QVariant(self.driver.VoltageUp2))
        self.resistor_combo_box.addItem("470 kom", QtCore.QVariant(self.driver.VoltageUp3))
        self.resistor_combo_box.addItem("Off", QtCore.QVariant(self.driver.VoltageOff))
        self.resistor_combo_box.addItem("Zero", QtCore.QVariant(self.driver.VoltageZero))
        self.resistor_combo_box.currentIndexChanged.connect(self.OnPullUpResistorChanged)

        resistor_hbox = QtGui.QHBoxLayout()
        resistor_hbox.addWidget(spin_label_resistor)
        resistor_hbox.addWidget(self.resistor_combo_box)
        resistor_hbox.addStretch(1)

        self.measure_button = QtGui.QPushButton("Measure")
        self.measure_button.clicked.connect(self.OnMeasureButton)

        self.czero_button = QtGui.QPushButton("Calibrate Czero")
        self.czero_button.clicked.connect(self.OnCalibrateCZero)

        left_vbox = QtGui.QVBoxLayout()
        left_vbox.addWidget(self.canvas)
        left_vbox.addWidget(self.mpl_toolbar)
        info_label = QtGui.QLabel(
                u'Для калибровки нуля емкости\n разомкните щупы.\n'+
                u'Для калибровки нуля напряжения\n замкните щупы.'
                )

        right_vbox = QtGui.QVBoxLayout()
        right_vbox.addLayout(connect_hbox)
        right_vbox.addLayout(resistor_hbox)
        right_vbox.addWidget(self.measure_button)
        right_vbox.addWidget(self.czero_button)
        right_vbox.addWidget(info_label)
        right_vbox.addStretch(1)
        
        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(left_vbox, stretch=1)
        hbox.addLayout(right_vbox)
        self.main_frame.setLayout(hbox)

        self.setCentralWidget(self.main_frame)
    
    def create_menu(self):        
        self.file_menu = self.menuBar().addMenu("&File")
        
        load_action = self.create_action("&Load file",
            shortcut="Ctrl+L", slot=self.load_file, tip="Load a file")
        save_action = self.create_action("&Save file",
            shortcut="Ctrl+S", slot=self.save_file, tip="Save a file")
        quit_action = self.create_action("&Quit", slot=self.close, 
            shortcut="Ctrl+Q", tip="Close the application")
        
        self.add_actions(self.file_menu, 
            (load_action, save_action, 
                None, quit_action))
            
        self.help_menu = self.menuBar().addMenu("&Help")
        about_action = self.create_action("&About", 
            shortcut='F1', slot=self.on_about, 
            tip='About the demo')
        
        self.add_actions(self.help_menu, (about_action,))

    def add_actions(self, target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)

    def create_action(  self, text, slot=None, shortcut=None, 
                        icon=None, tip=None, checkable=False, 
                        signal="triggered()"):
        action = QtGui.QAction(text, self)
        if icon is not None:
            action.setIcon(QIcon(":/%s.png" % icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            self.connect(action, QtCore.SIGNAL(signal), slot)
        if checkable:
            action.setCheckable(True)
        return action

    def OnConnectButton(self):
        try:
            self.driver.findDevice()
        except ValueError:
            self.connect_state.setStyleSheet("QLabel { color : red; }");
            self.connect_state.setText("Failed")
        else:
            self.connect_button.setText("Reconnect")
            self.connect_state.setStyleSheet("QLabel { color : green; }");
            self.connect_state.setText("Connected")
            
        pass
    def OnPullUpResistorChanged(self, index):
        print "OnPullUpResistorChanged", index
        pass

    def calibration(self):
        self.driver.conversion(self.driver.VoltageZero)
        values = self.driver.data.values
        mid = 0
        for v in values:
            mid+=v

        self.zero_value = mid/float(len(values))
        print "zero_value=", self.zero_value
        pass

    def OnMeasureButton(self):
        if not self.zero_value:
            self.calibration()

        voltage = self.resistor_combo_box.itemData(self.resistor_combo_box.currentIndex()).toInt()[0]
        self.driver.conversion(voltage)
        self.driver.data.zero_value = self.zero_value
        self.on_show()
        pass

    def MeasureMultiple(self, voltage):
        num_measure = 10
        datas = []
        min_values = None
        for num in xrange(num_measure):
            self.driver.conversion(voltage)
            time.sleep(0.2)
            self.driver.data.zero_value = self.zero_value

            len_values = len(self.driver.data.values)
            if not min_values:
                min_values = len_values
            else:
                if len_values<min_values:
                    min_values = len_values
            
            datas.append(self.driver.data)

        data = esr_data.ESRData()
        self.driver.data = data

        data.values = [0 for ix in xrange(min_values)]
        data.interval_ticks = 0
        for idata in xrange(num_measure):
            cur = datas[idata]
            for ix in xrange(min_values):
                data.values[ix] += cur.values[ix]
            data.interval_ticks += cur.interval_ticks*min_values/float(len(cur.values))

        for ix in xrange(min_values):
            data.values[ix] /= num_measure
        data.interval_ticks /= num_measure
        data.resistor_index = datas[0].resistor_index
        data.resistor_value = datas[0].resistor_value
        pass

    def OnMeasureMultiple(self):
        if not self.zero_value:
            self.calibration()

        voltage = self.resistor_combo_box.itemData(self.resistor_combo_box.currentIndex()).toInt()[0]
        self.driver.conversion(voltage)
        self.driver.data.zero_value = self.zero_value

        self.on_show()
        pass

    def OnCalibrateCZero(self):
        if not self.zero_value:
            self.zero_value = 0
            #self.calibration()

        voltage = self.driver.VoltageUp3
        num_measure = 10
        Carray = []
        Csum = 0
        for n in xrange(num_measure):
            time.sleep(0.2)
            self.driver.conversion(voltage)
            self.driver.data.zero_value = self.zero_value
            voltage_arr = self.driver.data.getVoltagesMillivolts()
            time_arr = self.driver.data.getTimesMicroseconds()

            C = self.calculate_linear(time_arr, voltage_arr)
            Carray.append(C)
            Csum += C

        Cmid = Csum/num_measure
        print "Czero=", "{:3.1f} pF".format(C*1e6)
        self.Czero = Cmid
        pass

def main():
    app = QtGui.QApplication(sys.argv)
    form = Form()
    form.show()
    app.exec_()


if __name__ == "__main__":
    main()
    
