# coding=UTF-8
import usb
import time
import datetime
import os

dev = None

VoltageOff = 0
VoltageZero = 1
VoltageUp1 = 2 #220 om 14 mA
VoltageUp2 = 3 #10 kom 0.33 mA
VoltageUp3 = 4 #470 ком 7 mkA

COMMAND_SET_VOLTAGE = 1
COMMAND_GET_ADC = 2
COMMAND_CONVERSION = 3
COMMAND_GET_RESULT = 4



class ReqType(object):
    VENDOR_REQUEST = 0x40
    RECIPIENT_ENDPOINT = 0x02
    RECIPIENT_DEVICE = 0
    DEVICE_TO_HOST = 0x80
    HOST_TO_DEVICE = 0x00

    #CTRL_OUT = RECIPIENT_ENDPOINT | VENDOR_REQUEST | HOST_TO_DEVICE
    #CTRL_IN  = RECIPIENT_ENDPOINT | VENDOR_REQUEST | DEVICE_TO_HOST
    CTRL_OUT = RECIPIENT_DEVICE | VENDOR_REQUEST | HOST_TO_DEVICE
    CTRL_IN  = RECIPIENT_DEVICE | VENDOR_REQUEST | DEVICE_TO_HOST

def findDevice():
    global dev
    dev = usb.core.find(idVendor=0x16C0, idProduct=0x05DC)

    if dev is None:
    	raise ValueError('Device not found')
    else:
        print "Device found"

    dev.set_configuration()

def readAll():
    time.sleep(0.1)
    while True:
        try:
            data = dev.read(129, 128, interface=1, timeout=50)
            if len(data)==0:
                break
            print "readb=", data
            print "read=", data.tostring()
        except usb.core.USBError:
            break
    pass

def readOne():
    time.sleep(0.1)
    try:
        data = dev.read(129, 128, interface=1, timeout=50)
        if len(data)==0:
            print "Read empty"
        else:
            print "readb=", data
    except usb.core.USBError:
        print "Read USB error"
    pass

def readInt():
    data = dev.read(129, 128, interface=1, timeout=50)
    if len(data)!=4:
        print "Fail read int len="+str(len(data))
    else:
        print data[0]+data[1]*0x100+data[2]*0x10000+data[3]*0x1000000

def readCommand():
    time.sleep(0.1)
    try:
        data = dev.read(129, 128, interface=1, timeout=50)
        if len(data)==0:
            print "Read empty"
            return
        else:
            print "readb=", data
    except usb.core.USBError:
        print "Read USB error"
        return

    if data[0]==COMMAND_SET_VOLTAGE:
        print "Voltage channel"+str(data[1]) 
    elif data[0]==COMMAND_GET_ADC:
        v = data[1]+256*data[2]
        print "voltage="+str(v*3.3/4096.0)
    elif data[0]==COMMAND_CONVERSION:
        print "Start conversion"
    else:
        print "Unknown command="+str(data[0])
    pass

def readConversionData():
    with open("out.dat", "wb") as file:
        step = 30
        for i in xrange(0, 34):
            offset = i*30
            print "write=",dev.write(3, [COMMAND_GET_RESULT, offset%256, offset//256], interface=1)
            data = dev.read(129, 128, interface=1, timeout=50)
            values =data[4:]
            print values
            file.write(values)

    pass

def printEndpoint(e):
    print "Endpoint:"
    print "bLength=", e.bLength
    print "bDescriptorType=", e.bDescriptorType
    print "bEndpointAddress=", e.bEndpointAddress
    print "bmAttributes=", e.bmAttributes
    print "wMaxPacketSize=", e.wMaxPacketSize
    print "bInterval=", e.bInterval
    print "bRefresh=", e.bRefresh
    print "bSynchAddress=", e.bSynchAddress
    pass

def main():
    findDevice()
    dev.set_configuration()
    for cfg in dev:
        for i in cfg:
            print "interface=", i.bInterfaceNumber
            for e in i:
                print e.bEndpointAddress
                #printEndpoint(e)

    if False:
        #readAll()
        print "write=",dev.write(3, "Hello, World!1KK", interface=1)
        readAll()
        #readAll()
        print "write=",dev.write(3, "0AXB", interface=1)
        readAll()
        #time.sleep(1)
        print "write=",dev.write(3, "B0RT", interface=1)
        readAll()
        print "write=",dev.write(3, "CMNK", interface=1)
        readAll()
        print "write=",dev.write(3, "DMNK", interface=1)
        #time.sleep(1)
        readAll()

    if False:
        readAll()
        print "write=",dev.write(3, "0", interface=1)
        readAll()
        print "write=",dev.write(3, "1", interface=1)
        readAll()

    if False:
        #print "write=",dev.write(3, "Hello, World!1KK", interface=1)
        #print "write=",dev.write(3, "Hello, World!2KK", interface=1)
        for i in xrange(1,10):
            #print i
            readInt()
    if False:
        readOne()
        i = VoltageUp2
        print "write=",dev.write(3, [COMMAND_SET_VOLTAGE, VoltageUp1], interface=1)
        readCommand()
        print "write=",dev.write(3, [COMMAND_GET_ADC], interface=1)
        readCommand()
        time.sleep(1)
        readOne()

    if True:
        readOne()
        print "write=",dev.write(3, [COMMAND_SET_VOLTAGE, VoltageZero], interface=1)
        readCommand()
        print "write=",dev.write(3, [COMMAND_CONVERSION, VoltageUp3], interface=1)
        readCommand()
        time.sleep(1)
        readConversionData()

    pass


main()

